def get_recursively(search_dict, field):
    """
    Takes a dict with nested lists and dicts,
    and searches all dicts for a key of the field
    provided.
    """
    fields_found = []

    for key, value in search_dict.items():

        if key == field:
            fields_found.append(value)

        elif isinstance(value, dict):
            results = get_recursively(value, field)
            for result in results:
                fields_found.append(result)

        elif isinstance(value, list):
            for item in value:
                if isinstance(item, dict):
                    more_results = get_recursively(item, field)
                    for another_result in more_results:
                        fields_found.append(another_result)

    return fields_found



import datetime
import stripe

from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework import status

from payments.payments.recursive_searcher import get_recursively

OLD_SUB = None
HOW_MUCH_MONEY = None


@api_view(['POST'])
def create_sub(request):
    # here we retrieve the customer made in create_customer_and_payment_method
    # for creating a sub
    customer_id = [x for x in stripe.Customer.list(limit=3)["data"]
                   if x["description"] == "Test Customer"][0]["id"]

    # here we retrieve a product id for retrieving a price id
    product_id = [x for x in stripe.Product.list(limit=2)["data"]
                  if x["name"] == "Silver Advance"][0]["id"]
    price_id = [x for x in stripe.Price.list(limit=2000)["data"]
                if x["product"] == product_id][0]["id"]

    # here we create the sub based on customer and price ids
    subscription = stripe.Subscription.create(
        customer=customer_id,
        items=[
            {"price": price_id},
        ],
    )

    return Response(status=status.HTTP_204_NO_CONTENT, data=subscription)


@api_view(['POST'])
def cancel_subscription(request):
    try:
        # here we hardcode retrieve a subscription
        sub = stripe.Subscription.retrieve(
            stripe.Subscription.list(limit=1, status="active")
            ["data"][0]["id"],
        )
        remove_sub = stripe.Subscription.delete(
            sub,
        )
        return Response(status=status.HTTP_204_NO_CONTENT, data=remove_sub)
    except Exception as e:
        print(e)
        remove_sub = {}
        return Response(status=status.HTTP_404_NOT_FOUND, data=remove_sub)


@api_view(['GET'])
def subscription_list(request):
    """
        STATUSES:
        active
        past_due
        unpaid
        canceled
        xc
        incomplete_expired
        trialing
        all
        ended
        Here we can check for example unpaid subs or incomplete due to
        3d secure security.
    """
    return Response(status=status.HTTP_200_OK,
                    data=stripe.Subscription.list(limit=300, status="active",
                                                  customer="cus_KwBZaypcmQkq7w"))


@api_view(['POST'])
def subscription_alteration(request):
    global OLD_SUB, HOW_MUCH_MONEY
    """
        In this function we calculate how much money we need for refunding,
        we cancel the sub, refund the customer, activate a new subscription.
    """

    customer_id = [x for x in stripe.Customer.list(limit=3)["data"]
                   if x["description"] == "Test Customer"][0]["id"]

    charge = stripe.Charge.list(limit=10)["data"][6] if \
        stripe.Charge.list(limit=5)["data"][0]["customer"] \
        == customer_id else None

    if datetime.datetime.now().day in [x for x in range(1, 6)]:
        HOW_MUCH_MONEY = int(charge["amount"] * 1)
    elif datetime.datetime.now().day in [x for x in range(6, 11)]:
        HOW_MUCH_MONEY = int(charge["amount"] * 0.5)
    else:
        HOW_MUCH_MONEY = 0

    # here we cancel the old sub and the last invoice generated by the system
    old_product = [x for x in stripe.Product.list(limit=10)["data"]
                   if x["name"] == "Silver Advance"][0]["id"]
    for sub in stripe.Subscription.list(limit=10)["data"]:
        if old_product in get_recursively(sub, "product"):
            OLD_SUB = sub
            break
    delete_sub = stripe.Subscription.delete(
        OLD_SUB["id"],
    )

    # we can only refund a customer if sum is greater then 1
    if HOW_MUCH_MONEY > 1:
        refund = stripe.Refund.create(
            charge=charge["id"],
            amount=HOW_MUCH_MONEY,
            reason="requested_by_customer"
        )

    # here we retrieve a product id for retrieving a price id
    product_id = [x for x in stripe.Product.list(limit=2)["data"]
                  if x["name"] == "Gold Special"][0]["id"]
    price_id = [x for x in stripe.Price.list(limit=2000)["data"]
                if x["product"] == product_id][0]["id"]

    subscription = stripe.Subscription.create(
        customer=customer_id,
        items=[
            {"price": price_id},
        ],
    )

    return Response(status=status.HTTP_204_NO_CONTENT, data=subscription)



import datetime
import stripe

from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework import status

KEY = stripe.api_key = 'sk_test_51K70bDJrktwvdD84D4Y45kSYHKA0dHvfb' \
                       'on0Xnog4Ib54emyy44jbgEJlNqv6f7KV99UkSc7ss7RYid' \
                       'EBdXgMwef00oT3RSCrU'


@api_view(['POST'])
def create_customer_payment_method(request):
    # here we create a new customer
    test_customer = stripe.Customer.create(
        description="Test Customer",
        email="test@test.ro",
    )
    # here we create a new payment method
    payment_method_create_card = stripe.PaymentMethod.create(
        type="card",
        card={
            "number": "4242424242424242",
            "exp_month": 12,
            "exp_year": 2022,
            "cvc": "314",
        },
    )

    # here we attach the payment method for the customer
    stripe.PaymentMethod.attach(
        payment_method_create_card.stripe_id,
        customer=test_customer.stripe_id,
    )

    # here we modify the default payment method for test_customer
    stripe.Customer.modify(
        test_customer.stripe_id,
        invoice_settings={"default_payment_method":
                          payment_method_create_card.stripe_id},
    )

    return Response(status=status.HTTP_200_OK, data=test_customer)


@api_view(['POST'])
def create_products(request):
    """
        Here we create two products for subscription usage
    """
    # here we create the first product for future subscriptions
    product_gold = stripe.Product.create(
        name="Gold Special",
    )
    # here we create a price and other data for the first product
    stripe.Price.create(
        unit_amount=2000,
        currency="ron",
        recurring={"interval": "month"},
        product=product_gold.stripe_id,
    )

    # here we create the second product for future subscriptions
    product_silver = stripe.Product.create(
        name="Silver Advance",
    )
    # here we create a price and other data for the second product
    stripe.Price.create(
        unit_amount=1000,
        currency="ron",
        recurring={"interval": "month"},
        product=product_silver.stripe_id,
    )
    return Response(status=status.HTTP_200_OK,
                    data=product_gold | product_silver)


def refund_helper(charge, percent):
    """
        This function is a helper for refund_customer.
        We create a Refund object based on charge object and percent.
    """
    refund = stripe.Refund.create(
        charge=charge["id"],
        amount=int(charge["amount"] * percent),
        reason="requested_by_customer"
    )
    return Response(status=status.HTTP_204_NO_CONTENT, data=refund)


@api_view(['POST'])
def refund_customer(request):
    """
        This function refunds the customer based on how many days
         have passed since the begging on the month.
         We refund the customer as it follows:
         1-5 100% $
         6-10 75% $
         15-17 50% $
         17-end 0% $
    """

    customer_id = [x for x in stripe.Customer.list(limit=3)["data"]
                   if x["description"] == "Test Customer"][0]["id"]

    charge = stripe.Charge.list(limit=3)["data"][2] if\
        stripe.Charge.list(limit=3)["data"][0]["customer"]\
        == customer_id else None

    if datetime.datetime.now().day in [x for x in range(1, 6)]:
        return refund_helper(charge, 1)
    elif datetime.datetime.now().day in [x for x in range(6, 11)]:
        return refund_helper(charge, 0.75)
    elif datetime.datetime.now().day in [x for x in range(15, 18)]:
        return refund_helper(charge, 0.5)
    else:
        return Response(
            status=status.HTTP_403_FORBIDDEN, data={
                "refund": "we can only refund a customer if"
                          " sum is greater then 1"})


@api_view(['POST'])
def change_card_details(request):
    customer_id = "cus_KnH5alP1AzYqDL"
    customer_object = stripe.Customer.retrieve(customer_id)
    if not stripe.Subscription.list(limit=300, status="unpaid",
                                    customer=customer_id):
        card = stripe.Customer.retrieve_source(
            customer_id,
            "card_1K7gukJrktwvdD84JBn6UFWw",
        )


@api_view(['GET'])
def check_unpaid_invoices_customer(request):
    customer_id = [x for x in stripe.Customer.list(limit=3)["data"]
                   if x["description"] == "Test Customer"][0]["id"]
    customer_invoices = stripe.Invoice.list(limit=200, customer=customer_id,
                                            status='uncollectible')
    return Response(status=200, data=customer_invoices)



from django.urls import path

from payments.subscriptions import views

urlpatterns = [
    path('create-sub/', views.create_sub),
    path('cancel-subscription/', views.cancel_subscription),
    path('subscription-list/', views.subscription_list),
    path('subscription-alteration/', views.subscription_alteration),
]



import stripe
import logging

from rest_framework.response import Response
from rest_framework.decorators import api_view


ENDPOINT_SECRET = 'whsec_BoDAsE6ruehGJAN78uSktD7pGJMMkBOJ'

logger = logging.getLogger(__name__)


@api_view(['POST'])
def events(request):
    """
        stripe listen --forward-to http://127.0.0.1:8000/
            webhooks/events/
        stripe trigger customer.created(to mock this action)
        SEE:
        https://stripe.com/docs/api/events/types
    """
    payload = request.body
    sig_header = request.META['HTTP_STRIPE_SIGNATURE']

    try:
        event = stripe.Webhook.construct_event(
            payload, sig_header, ENDPOINT_SECRET
        )
        events = stripe.Event.list()
        # print(events)
    except ValueError as e:
        logger.info(e)
        # Invalid payload
        return Response(status=400)
    except stripe.error.SignatureVerificationError as e:
        logger.info(e)
        # Invalid signature
        return Response(status=400)

    # Handle the event, from here we can send an email, store in the db
    # some information
    if event.type == 'customer.created':
        # Occurs whenever a new customer is created.
        customer = event.data.object
        logger.info(customer)
    if event.type == 'issuing_card.created':
        # Occurs whenever a card is created.
        card = event.data.object
        logger.info(card)
    if event.type == 'invoice.payment_failed':
        # Occurs whenever an invoice payment attempt fails,
        # due either to a declined payment or to the lack of a
        # stored payment method
        unpaid_sub = event.data.object
        logger.info(unpaid_sub)
    if event.type == 'customer.source.created':
        # Occurs whenever a new source(a customer's payment instrument)
        # is created for a customer.
        money_source = event.data.object
        logger.info(money_source)
    if event.type == 'payment_intent.succeeded':
        payment_intent = event.data.object  # contains a stripe.PaymentIntent
        logger.info(payment_intent)
    if event.type == 'payment_method.attached':
        payment_method = event.data.object  # contains a stripe.PaymentMethod
        logger.info(payment_method)
    # ... handle other event types
    else:
        logger.info('Unhandled event type {}'.format(event.type))

    return Response(status=200, data=events)



from django.urls import path

from payments.payments import views

urlpatterns = [
    path('create-products/', views.create_products),
    path('create-customer/', views.create_customer_payment_method),
    path('change-card-details/', views.change_card_details),
    path('refund/', views.refund_customer),
    path('check-unpaid-invoices/', views.check_unpaid_invoices_customer),
]
